;; Problem 35
;; 17 January 2003
;;
;; The number, 197, is called a circular prime because all rotations of
;; the digits: 197, 971, and 719, are themselves prime.
;;
;; There are thirteen such primes below 100: 2, 3, 5, 7, 11, 13, 17, 31,
;; 37, 71, 73, 79, and 97.
;;
;; How many circular primes are there below one million?

(defvar *some-primes*
  '(2 3 5 7 11 13 17 19))

(defvar *max-some-primes*
  (apply #'max *some-primes*))

(defun add-cached-prime (prime)
  (when (not (member prime *some-primes*))
    (push prime *some-primes*)
    (setf *max-some-primes* (apply #'max *some-primes*))))

(defun reset-cache ()
  (setf *max-some-primes* 2)
  (setf *some-primes* '(2 3 5)))

(defun range (start end)
  (loop for nn from start to end
	collect nn))

(defun prime? (num)
  (when (= 2 num)
    (return-from prime? t))
  (loop for fact = (ceiling (sqrt num))
	then (if (oddp fact)
		 (- fact 2)
		 (- fact 1))
	while (> fact 1)
	do
	(when (= 0 (mod num fact))
	  (return-from prime? nil)))
  (not (= 0 (mod num 2))))


(assert (not (prime? 92)))
(assert      (prime? 2))
(assert      (prime? 3))
(assert (not (prime? 4)))
(assert      (prime? 5))
(assert (not (prime? 6)))


(defmacro aprog1 (first &body body)
  `(let ((it ,first))
    (progn
      ,@body)
    it))

(defun make-prime-spigot ()
  (let ((cached-primes *some-primes*)
	(last-num nil))
    (sort cached-primes #'<)
    (lambda ()
      (cond ((not (null cached-primes))
	     (aprog1
		 (pop cached-primes)
	       (setf last-num it)))
	    (t
	     (loop for next = (+ last-num 2)
		   then (+ next 2)
		   do
		   (when (prime? next)
		     (setf last-num next)
		     (return next))))))))

'(loop for ii from 1 to 100
      do
      (format t "ii:~a : ~a~&"
	      ii 
	      (prime? ii)))

(defun vector->list (vec)
  (loop for item across vec
	collect item))

(defun list->vector (lst)
  (let ((vec (make-array (length lst))))
    (loop for idx = 0 then (+ 1 idx)
	  for item in lst
	  do
	  (setf (aref vec idx) item))
    vec))

; (vector->list #(1 2 3))
; (list->vector (vector->list #(1 2 3)))

(defun explode-string (str)
  (loop for ch across str
	collect (format nil "~a" ch)))

; (explode-string "abc")

(defun all-rotations (num)
  (let ((nlist (mapcar #'read-from-string (explode-string (format nil "~d" num)))))
    (loop for ii from 1 to (length nlist)
	  for permuted = (copy-list nlist) then (copy-list permuted)
	  collect (setf permuted
			(append (rest permuted)
				(list (first permuted)))))))

(defun all-rotations->numbers (num)
  (mapcar #'(lambda (rotation)
	      (read-from-string
	       (format nil "~{~d~}" rotation)))
	  (all-rotations num)))

;(all-rotations->numbers 197)

(defun all-rotations-prime (num)
  (let ((result t))
    (loop for rotation in (all-rotations->numbers num)
	  do
;; 	  (format t "trying:~a => ~a ? ~a~&"
;; 		  num
;; 		  rotation
;; 		  (prime? rotation))
	  (when (not (prime? rotation))
	    (return-from all-rotations-prime nil)))
    result))

(assert (all-rotations-prime 2))
(assert (all-rotations-prime 197))
(assert (not (all-rotations-prime 92)))

(defun primes-under (num)
  (let ((res (list 2)))
    (loop for ii = 3 then (+ ii 2)
	  do
	  (when (> ii num)
	    (return t))
	  (when (prime? ii)
	    (push ii res)))
    (reverse res)))

(defvar *primes-under-a-million* (primes-under 1000000))

;; (with-open-file (out "/home/mortis/personal/projects/project-euler/prms.txt"
;; 		     :direction :output
;; 		     :if-exists :supersede)
;;   (loop for num in *primes-under-a-million*
;; 	do
;; 	(format out "~d~&" num)))

(assert (equalp
	 '(2 3 5 7 11 13 17 31 37 71 73 79 97)
	 (sort 
	  (let ((res (list)))
	    (loop for num from 2 to 99
		  do
		  (when (all-rotations-prime num)
		    (push num res)
		    (format t "ok: ~d~&" num)))
	    res)
	  #'<)))



(time
 (let ((num-circular 0))
   (loop for prime in *primes-under-a-million*
	 do
	 (when (all-rotations-prime prime)
	   (incf num-circular)))
   (format t "num-circular:~a~&" num-circular)
   num-circular))
